@@ -1,18 +1,22 @@
-
 /* strop module */
 
-static char strop_module__doc__[] =
-"Common string manipulations, optimized for speed.\n\
-\n\
-Always use \"import string\" rather than referencing\n\
-this module directly.";
-
+#define PY_SSIZE_T_CLEAN
 #include "Python.h"
-
 #include <ctype.h>
+
+PyDoc_STRVAR(strop_module__doc__,
+"Common string manipulations, optimized for speed.\n"
+"\n"
+"Always use \"import string\" rather than referencing\n"
+"this module directly.");
+
 /* XXX This file assumes that the <ctype.h> is*() functions
    XXX are defined for all 8-bit characters! */
 
+#define WARN if (PyErr_Warn(PyExc_DeprecationWarning, \
+		       "strop functions are obsolete; use string methods")) \
+	     return NULL
+
 /* The lstrip(), rstrip() and strip() functions are implemented
    in do_strip(), which uses an additional parameter to indicate what
    type of strip should occur. */
@@ -23,10 +27,11 @@ this module directly.";
 
 
 static PyObject *
-split_whitespace(char *s, int len, int maxsplit)
+split_whitespace(char *s, Py_ssize_t len, Py_ssize_t maxsplit)
 {
-	int i = 0, j, err;
-	int countsplit = 0;
+	Py_ssize_t i = 0, j;
+	int err;
+	Py_ssize_t countsplit = 0;
 	PyObject* item;
 	PyObject *list = PyList_New(0);
 
@@ -42,7 +47,7 @@ split_whitespace(char *s, int len, int maxsplit)
 			i = i+1;
 		}
 		if (j < i) {
-			item = PyString_FromStringAndSize(s+j, (int)(i-j));
+			item = PyString_FromStringAndSize(s+j, i-j);
 			if (item == NULL)
 				goto finally;
 
@@ -57,7 +62,7 @@ split_whitespace(char *s, int len, int maxsplit)
 			}
 			if (maxsplit && (countsplit >= maxsplit) && i < len) {
 				item = PyString_FromStringAndSize(
-                                        s+i, (int)(len - i));
+                                        s+i, len - i);
 				if (item == NULL)
 					goto finally;
 
@@ -77,30 +82,31 @@ split_whitespace(char *s, int len, int maxsplit)
 }
 
 
-static char splitfields__doc__[] =
-"split(s [,sep [,maxsplit]]) -> list of strings\n\
-splitfields(s [,sep [,maxsplit]]) -> list of strings\n\
-\n\
-Return a list of the words in the string s, using sep as the\n\
-delimiter string.  If maxsplit is nonzero, splits into at most\n\
-maxsplit words.  If sep is not specified, any whitespace string\n\
-is a separator.  Maxsplit defaults to 0.\n\
-\n\
-(split and splitfields are synonymous)";
+PyDoc_STRVAR(splitfields__doc__,
+"split(s [,sep [,maxsplit]]) -> list of strings\n"
+"splitfields(s [,sep [,maxsplit]]) -> list of strings\n"
+"\n"
+"Return a list of the words in the string s, using sep as the\n"
+"delimiter string.  If maxsplit is nonzero, splits into at most\n"
+"maxsplit words.  If sep is not specified, any whitespace string\n"
+"is a separator.  Maxsplit defaults to 0.\n"
+"\n"
+"(split and splitfields are synonymous)");
 
 static PyObject *
 strop_splitfields(PyObject *self, PyObject *args)
 {
-	int len, n, i, j, err;
-	int splitcount, maxsplit;
+	Py_ssize_t len, n, i, j, err;
+	Py_ssize_t splitcount, maxsplit;
 	char *s, *sub;
 	PyObject *list, *item;
 
+	WARN;
 	sub = NULL;
 	n = 0;
 	splitcount = 0;
 	maxsplit = 0;
-	if (!PyArg_ParseTuple(args, "t#|z#i:split", &s, &len, &sub, &n, &maxsplit))
+	if (!PyArg_ParseTuple(args, "t#|z#n:split", &s, &len, &sub, &n, &maxsplit))
 		return NULL;
 	if (sub == NULL)
 		return split_whitespace(s, len, maxsplit);
@@ -116,7 +122,7 @@ strop_splitfields(PyObject *self, PyObject *args)
 	i = j = 0;
 	while (i+n <= len) {
 		if (s[i] == sub[0] && (n == 1 || memcmp(s+i, sub, n) == 0)) {
-			item = PyString_FromStringAndSize(s+j, (int)(i-j));
+			item = PyString_FromStringAndSize(s+j, i-j);
 			if (item == NULL)
 				goto fail;
 			err = PyList_Append(list, item);
@@ -131,7 +137,7 @@ strop_splitfields(PyObject *self, PyObject *args)
 		else
 			i++;
 	}
-	item = PyString_FromStringAndSize(s+j, (int)(len-j));
+	item = PyString_FromStringAndSize(s+j, len-j);
 	if (item == NULL)
 		goto fail;
 	err = PyList_Append(list, item);
@@ -147,27 +153,28 @@ strop_splitfields(PyObject *self, PyObject *args)
 }
 
 
-static char joinfields__doc__[] =
-"join(list [,sep]) -> string\n\
-joinfields(list [,sep]) -> string\n\
-\n\
-Return a string composed of the words in list, with\n\
-intervening occurrences of sep.  Sep defaults to a single\n\
-space.\n\
-\n\
-(join and joinfields are synonymous)";
+PyDoc_STRVAR(joinfields__doc__,
+"join(list [,sep]) -> string\n"
+"joinfields(list [,sep]) -> string\n"
+"\n"
+"Return a string composed of the words in list, with\n"
+"intervening occurrences of sep.  Sep defaults to a single\n"
+"space.\n"
+"\n"
+"(join and joinfields are synonymous)");
 
 static PyObject *
 strop_joinfields(PyObject *self, PyObject *args)
 {
 	PyObject *seq;
 	char *sep = NULL;
-	int seqlen, seplen = 0;
-	int i, reslen = 0, slen = 0, sz = 100;
+	Py_ssize_t seqlen, seplen = 0;
+	Py_ssize_t i, reslen = 0, slen = 0, sz = 100;
 	PyObject *res = NULL;
 	char* p = NULL;
-	intargfunc getitemfunc;
+	ssizeargfunc getitemfunc;
 
+	WARN;
 	if (!PyArg_ParseTuple(args, "O|t#:join", &seq, &sep, &seplen))
 		return NULL;
 	if (sep == NULL) {
@@ -210,10 +217,8 @@ strop_joinfields(PyObject *self, PyObject *args)
 			}
 			slen = PyString_GET_SIZE(item);
 			while (reslen + slen + seplen >= sz) {
-				if (_PyString_Resize(&res, sz * 2)) {
-					Py_DECREF(res);
+				if (_PyString_Resize(&res, sz * 2) < 0)
 					return NULL;
-				}
 				sz *= 2;
 				p = PyString_AsString(res) + reslen;
 			}
@@ -226,10 +231,7 @@ strop_joinfields(PyObject *self, PyObject *args)
 			p += slen;
 			reslen += slen;
 		}
-		if (_PyString_Resize(&res, reslen)) {
-			Py_DECREF(res);
-			res = NULL;
-		}
+		_PyString_Resize(&res, reslen);
 		return res;
 	}
 
@@ -252,8 +254,7 @@ strop_joinfields(PyObject *self, PyObject *args)
 		}
 		slen = PyString_GET_SIZE(item);
 		while (reslen + slen + seplen >= sz) {
-			if (_PyString_Resize(&res, sz * 2)) {
-				Py_DECREF(res);
+			if (_PyString_Resize(&res, sz * 2) < 0) {
 				Py_DECREF(item);
 				return NULL;
 			}
@@ -270,30 +271,28 @@ strop_joinfields(PyObject *self, PyObject *args)
 		reslen += slen;
 		Py_DECREF(item);
 	}
-	if (_PyString_Resize(&res, reslen)) {
-		Py_DECREF(res);
-		res = NULL;
-	}
+	_PyString_Resize(&res, reslen);
 	return res;
 }
 
 
-static char find__doc__[] =
-"find(s, sub [,start [,end]]) -> in\n\
-\n\
-Return the lowest index in s where substring sub is found,\n\
-such that sub is contained within s[start,end].  Optional\n\
-arguments start and end are interpreted as in slice notation.\n\
-\n\
-Return -1 on failure.";
+PyDoc_STRVAR(find__doc__,
+"find(s, sub [,start [,end]]) -> in\n"
+"\n"
+"Return the lowest index in s where substring sub is found,\n"
+"such that sub is contained within s[start,end].  Optional\n"
+"arguments start and end are interpreted as in slice notation.\n"
+"\n"
+"Return -1 on failure.");
 
 static PyObject *
 strop_find(PyObject *self, PyObject *args)
 {
 	char *s, *sub;
-	int len, n, i = 0, last = INT_MAX;
+	Py_ssize_t len, n, i = 0, last = PY_SSIZE_T_MAX;
 
-	if (!PyArg_ParseTuple(args, "t#t#|ii:find", &s, &len, &sub, &n, &i, &last))
+	WARN;
+	if (!PyArg_ParseTuple(args, "t#t#|nn:find", &s, &len, &sub, &n, &i, &last))
 		return NULL;
 
 	if (last > len)
@@ -320,23 +319,24 @@ strop_find(PyObject *self, PyObject *args)
 }
 
 
-static char rfind__doc__[] =
-"rfind(s, sub [,start [,end]]) -> int\n\
-\n\
-Return the highest index in s where substring sub is found,\n\
-such that sub is contained within s[start,end].  Optional\n\
-arguments start and end are interpreted as in slice notation.\n\
-\n\
-Return -1 on failure.";
+PyDoc_STRVAR(rfind__doc__,
+"rfind(s, sub [,start [,end]]) -> int\n"
+"\n"
+"Return the highest index in s where substring sub is found,\n"
+"such that sub is contained within s[start,end].  Optional\n"
+"arguments start and end are interpreted as in slice notation.\n"
+"\n"
+"Return -1 on failure.");
 
 static PyObject *
 strop_rfind(PyObject *self, PyObject *args)
 {
 	char *s, *sub;
-	int len, n, j;
-	int i = 0, last = INT_MAX;
+	Py_ssize_t len, n, j;
+	Py_ssize_t i = 0, last = PY_SSIZE_T_MAX;
 
-	if (!PyArg_ParseTuple(args, "t#t#|ii:rfind", &s, &len, &sub, &n, &i, &last))
+	WARN;
+	if (!PyArg_ParseTuple(args, "t#t#|nn:rfind", &s, &len, &sub, &n, &i, &last))
 		return NULL;
 
 	if (last > len)
@@ -366,10 +366,10 @@ static PyObject *
 do_strip(PyObject *args, int striptype)
 {
 	char *s;
-	int len, i, j;
+	Py_ssize_t len, i, j;
 
 
-	if (!PyArg_Parse(args, "t#", &s, &len))
+	if (PyString_AsStringAndSize(args, &s, &len))
 		return NULL;
 
 	i = 0;
@@ -396,62 +396,66 @@ do_strip(PyObject *args, int striptype)
 }
 
 
-static char strip__doc__[] =
-"strip(s) -> string\n\
-\n\
-Return a copy of the string s with leading and trailing\n\
-whitespace removed.";
+PyDoc_STRVAR(strip__doc__,
+"strip(s) -> string\n"
+"\n"
+"Return a copy of the string s with leading and trailing\n"
+"whitespace removed.");
 
 static PyObject *
 strop_strip(PyObject *self, PyObject *args)
 {
+	WARN;
 	return do_strip(args, BOTHSTRIP);
 }
 
 
-static char lstrip__doc__[] =
-"lstrip(s) -> string\n\
-\n\
-Return a copy of the string s with leading whitespace removed.";
+PyDoc_STRVAR(lstrip__doc__,
+"lstrip(s) -> string\n"
+"\n"
+"Return a copy of the string s with leading whitespace removed.");
 
 static PyObject *
 strop_lstrip(PyObject *self, PyObject *args)
 {
+	WARN;
 	return do_strip(args, LEFTSTRIP);
 }
 
 
-static char rstrip__doc__[] =
-"rstrip(s) -> string\n\
-\n\
-Return a copy of the string s with trailing whitespace removed.";
+PyDoc_STRVAR(rstrip__doc__,
+"rstrip(s) -> string\n"
+"\n"
+"Return a copy of the string s with trailing whitespace removed.");
 
 static PyObject *
 strop_rstrip(PyObject *self, PyObject *args)
 {
+	WARN;
 	return do_strip(args, RIGHTSTRIP);
 }
 
 
-static char lower__doc__[] =
-"lower(s) -> string\n\
-\n\
-Return a copy of the string s converted to lowercase.";
+PyDoc_STRVAR(lower__doc__,
+"lower(s) -> string\n"
+"\n"
+"Return a copy of the string s converted to lowercase.");
 
 static PyObject *
 strop_lower(PyObject *self, PyObject *args)
 {
 	char *s, *s_new;
-	int i, n;
-	PyObject *new;
+	Py_ssize_t i, n;
+	PyObject *newstr;
 	int changed;
 
-	if (!PyArg_Parse(args, "t#", &s, &n))
+	WARN;
+	if (PyString_AsStringAndSize(args, &s, &n))
 		return NULL;
-	new = PyString_FromStringAndSize(NULL, n);
-	if (new == NULL)
+	newstr = PyString_FromStringAndSize(NULL, n);
+	if (newstr == NULL)
 		return NULL;
-	s_new = PyString_AsString(new);
+	s_new = PyString_AsString(newstr);
 	changed = 0;
 	for (i = 0; i < n; i++) {
 		int c = Py_CHARMASK(*s++);
@@ -463,33 +467,34 @@ strop_lower(PyObject *self, PyObject *args)
 		s_new++;
 	}
 	if (!changed) {
-		Py_DECREF(new);
+		Py_DECREF(newstr);
 		Py_INCREF(args);
 		return args;
 	}
-	return new;
+	return newstr;
 }
 
 
-static char upper__doc__[] =
-"upper(s) -> string\n\
-\n\
-Return a copy of the string s converted to uppercase.";
+PyDoc_STRVAR(upper__doc__,
+"upper(s) -> string\n"
+"\n"
+"Return a copy of the string s converted to uppercase.");
 
 static PyObject *
 strop_upper(PyObject *self, PyObject *args)
 {
 	char *s, *s_new;
-	int i, n;
-	PyObject *new;
+	Py_ssize_t i, n;
+	PyObject *newstr;
 	int changed;
 
-	if (!PyArg_Parse(args, "t#", &s, &n))
+	WARN;
+	if (PyString_AsStringAndSize(args, &s, &n))
 		return NULL;
-	new = PyString_FromStringAndSize(NULL, n);
-	if (new == NULL)
+	newstr = PyString_FromStringAndSize(NULL, n);
+	if (newstr == NULL)
 		return NULL;
-	s_new = PyString_AsString(new);
+	s_new = PyString_AsString(newstr);
 	changed = 0;
 	for (i = 0; i < n; i++) {
 		int c = Py_CHARMASK(*s++);
@@ -501,34 +506,35 @@ strop_upper(PyObject *self, PyObject *args)
 		s_new++;
 	}
 	if (!changed) {
-		Py_DECREF(new);
+		Py_DECREF(newstr);
 		Py_INCREF(args);
 		return args;
 	}
-	return new;
+	return newstr;
 }
 
 
-static char capitalize__doc__[] =
-"capitalize(s) -> string\n\
-\n\
-Return a copy of the string s with only its first character\n\
-capitalized.";
+PyDoc_STRVAR(capitalize__doc__,
+"capitalize(s) -> string\n"
+"\n"
+"Return a copy of the string s with only its first character\n"
+"capitalized.");
 
 static PyObject *
 strop_capitalize(PyObject *self, PyObject *args)
 {
 	char *s, *s_new;
-	int i, n;
-	PyObject *new;
+	Py_ssize_t i, n;
+	PyObject *newstr;
 	int changed;
 
-	if (!PyArg_Parse(args, "t#", &s, &n))
+	WARN;
+	if (PyString_AsStringAndSize(args, &s, &n))
 		return NULL;
-	new = PyString_FromStringAndSize(NULL, n);
-	if (new == NULL)
+	newstr = PyString_FromStringAndSize(NULL, n);
+	if (newstr == NULL)
 		return NULL;
-	s_new = PyString_AsString(new);
+	s_new = PyString_AsString(newstr);
 	changed = 0;
 	if (0 < n) {
 		int c = Py_CHARMASK(*s++);
@@ -549,21 +555,21 @@ strop_capitalize(PyObject *self, PyObject *args)
 		s_new++;
 	}
 	if (!changed) {
-		Py_DECREF(new);
+		Py_DECREF(newstr);
 		Py_INCREF(args);
 		return args;
 	}
-	return new;
+	return newstr;
 }
 
 
-static char expandtabs__doc__[] =
-"expandtabs(string, [tabsize]) -> string\n\
-\n\
-Expand tabs in a string, i.e. replace them by one or more spaces,\n\
-depending on the current column and the given tab size (default 8).\n\
-The column number is reset to zero after each newline occurring in the\n\
-string.  This doesn't understand other non-printing characters.";
+PyDoc_STRVAR(expandtabs__doc__,
+"expandtabs(string, [tabsize]) -> string\n"
+"\n"
+"Expand tabs in a string, i.e. replace them by one or more spaces,\n"
+"depending on the current column and the given tab size (default 8).\n"
+"The column number is reset to zero after each newline occurring in the\n"
+"string.  This doesn't understand other non-printing characters.");
 
 static PyObject *
 strop_expandtabs(PyObject *self, PyObject *args)
@@ -572,12 +578,13 @@ strop_expandtabs(PyObject *self, PyObject *args)
 	char* e;
 	char* p;
 	char* q;
-	int i, j;
+	Py_ssize_t i, j, old_j;
 	PyObject* out;
 	char* string;
-	int stringlen;
+	Py_ssize_t stringlen;
 	int tabsize = 8;
 
+	WARN;
 	/* Get arguments */
 	if (!PyArg_ParseTuple(args, "s#|i:expandtabs", &string, &stringlen, &tabsize))
 		return NULL;
@@ -588,12 +595,18 @@ strop_expandtabs(PyObject *self, PyObject *args)
 	}
 
 	/* First pass: determine size of output string */
-	i = j = 0; /* j: current column; i: total of previous lines */
+	i = j = old_j = 0; /* j: current column; i: total of previous lines */
 	e = string + stringlen;
 	for (p = string; p < e; p++) {
-		if (*p == '\t')
+		if (*p == '\t') {
 			j += tabsize - (j%tabsize);
-		else {
+			if (old_j > j) {
+				PyErr_SetString(PyExc_OverflowError,
+						"new string is too long");
+				return NULL;
+			}
+			old_j = j;
+		} else {
 			j++;
 			if (*p == '\n') {
 				i += j;
@@ -602,6 +615,11 @@ strop_expandtabs(PyObject *self, PyObject *args)
 		}
 	}
 
+	if ((i + j) < 0) {
+		PyErr_SetString(PyExc_OverflowError, "new string is too long");
+		return NULL;
+	}
+
 	/* Second pass: create output string and fill it */
 	out = PyString_FromStringAndSize(NULL, i+j);
 	if (out == NULL)
@@ -628,22 +646,23 @@ strop_expandtabs(PyObject *self, PyObject *args)
 }
 
 
-static char count__doc__[] =
-"count(s, sub[, start[, end]]) -> int\n\
-\n\
-Return the number of occurrences of substring sub in string\n\
-s[start:end].  Optional arguments start and end are\n\
-interpreted as in slice notation.";
+PyDoc_STRVAR(count__doc__,
+"count(s, sub[, start[, end]]) -> int\n"
+"\n"
+"Return the number of occurrences of substring sub in string\n"
+"s[start:end].  Optional arguments start and end are\n"
+"interpreted as in slice notation.");
 
 static PyObject *
 strop_count(PyObject *self, PyObject *args)
 {
 	char *s, *sub;
-	int len, n;
-	int i = 0, last = INT_MAX;
-	int m, r;
+	Py_ssize_t len, n;
+	Py_ssize_t i = 0, last = PY_SSIZE_T_MAX;
+	Py_ssize_t m, r;
 
-	if (!PyArg_ParseTuple(args, "t#t#|ii:count", &s, &len, &sub, &n, &i, &last))
+	WARN;
+	if (!PyArg_ParseTuple(args, "t#t#|nn:count", &s, &len, &sub, &n, &i, &last))
 		return NULL;
 	if (last > len)
 		last = len;
@@ -672,26 +691,27 @@ strop_count(PyObject *self, PyObject *args)
 }
 
 
-static char swapcase__doc__[] =
-"swapcase(s) -> string\n\
-\n\
-Return a copy of the string s with upper case characters\n\
-converted to lowercase and vice versa.";
+PyDoc_STRVAR(swapcase__doc__,
+"swapcase(s) -> string\n"
+"\n"
+"Return a copy of the string s with upper case characters\n"
+"converted to lowercase and vice versa.");
 
 static PyObject *
 strop_swapcase(PyObject *self, PyObject *args)
 {
 	char *s, *s_new;
-	int i, n;
-	PyObject *new;
+	Py_ssize_t i, n;
+	PyObject *newstr;
 	int changed;
 
-	if (!PyArg_Parse(args, "t#", &s, &n))
+	WARN;
+	if (PyString_AsStringAndSize(args, &s, &n))
 		return NULL;
-	new = PyString_FromStringAndSize(NULL, n);
-	if (new == NULL)
+	newstr = PyString_FromStringAndSize(NULL, n);
+	if (newstr == NULL)
 		return NULL;
-	s_new = PyString_AsString(new);
+	s_new = PyString_AsString(newstr);
 	changed = 0;
 	for (i = 0; i < n; i++) {
 		int c = Py_CHARMASK(*s++);
@@ -708,23 +728,23 @@ strop_swapcase(PyObject *self, PyObject *args)
 		s_new++;
 	}
 	if (!changed) {
-		Py_DECREF(new);
+		Py_DECREF(newstr);
 		Py_INCREF(args);
 		return args;
 	}
-	return new;
+	return newstr;
 }
 
 
-static char atoi__doc__[] =
-"atoi(s [,base]) -> int\n\
-\n\
-Return the integer represented by the string s in the given\n\
-base, which defaults to 10.  The string s must consist of one\n\
-or more digits, possibly preceded by a sign.  If base is 0, it\n\
-is chosen from the leading characters of s, 0 for octal, 0x or\n\
-0X for hexadecimal.  If base is 16, a preceding 0x or 0X is\n\
-accepted.";
+PyDoc_STRVAR(atoi__doc__,
+"atoi(s [,base]) -> int\n"
+"\n"
+"Return the integer represented by the string s in the given\n"
+"base, which defaults to 10.  The string s must consist of one\n"
+"or more digits, possibly preceded by a sign.  If base is 0, it\n"
+"is chosen from the leading characters of s, 0 for octal, 0x or\n"
+"0X for hexadecimal.  If base is 16, a preceding 0x or 0X is\n"
+"accepted.");
 
 static PyObject *
 strop_atoi(PyObject *self, PyObject *args)
@@ -734,6 +754,7 @@ strop_atoi(PyObject *self, PyObject *args)
 	long x;
 	char buffer[256]; /* For errors */
 
+	WARN;
 	if (!PyArg_ParseTuple(args, "s|i:atoi", &s, &base))
 		return NULL;
 
@@ -749,18 +770,20 @@ strop_atoi(PyObject *self, PyObject *args)
 		x = (long) PyOS_strtoul(s, &end, base);
 	else
 		x = PyOS_strtol(s, &end, base);
-	if (end == s || !isalnum(end[-1]))
+	if (end == s || !isalnum(Py_CHARMASK(end[-1])))
 		goto bad;
 	while (*end && isspace(Py_CHARMASK(*end)))
 		end++;
 	if (*end != '\0') {
   bad:
-		sprintf(buffer, "invalid literal for atoi(): %.200s", s);
+		PyOS_snprintf(buffer, sizeof(buffer),
+			      "invalid literal for atoi(): %.200s", s);
 		PyErr_SetString(PyExc_ValueError, buffer);
 		return NULL;
 	}
 	else if (errno != 0) {
-		sprintf(buffer, "atoi() literal too large: %.200s", s);
+		PyOS_snprintf(buffer, sizeof(buffer), 
+			      "atoi() literal too large: %.200s", s);
 		PyErr_SetString(PyExc_ValueError, buffer);
 		return NULL;
 	}
@@ -768,16 +791,16 @@ strop_atoi(PyObject *self, PyObject *args)
 }
 
 
-static char atol__doc__[] =
-"atol(s [,base]) -> long\n\
-\n\
-Return the long integer represented by the string s in the\n\
-given base, which defaults to 10.  The string s must consist\n\
-of one or more digits, possibly preceded by a sign.  If base\n\
-is 0, it is chosen from the leading characters of s, 0 for\n\
-octal, 0x or 0X for hexadecimal.  If base is 16, a preceding\n\
-0x or 0X is accepted.  A trailing L or l is not accepted,\n\
-unless base is 0.";
+PyDoc_STRVAR(atol__doc__,
+"atol(s [,base]) -> long\n"
+"\n"
+"Return the long integer represented by the string s in the\n"
+"given base, which defaults to 10.  The string s must consist\n"
+"of one or more digits, possibly preceded by a sign.  If base\n"
+"is 0, it is chosen from the leading characters of s, 0 for\n"
+"octal, 0x or 0X for hexadecimal.  If base is 16, a preceding\n"
+"0x or 0X is accepted.  A trailing L or l is not accepted,\n"
+"unless base is 0.");
 
 static PyObject *
 strop_atol(PyObject *self, PyObject *args)
@@ -787,6 +810,7 @@ strop_atol(PyObject *self, PyObject *args)
 	PyObject *x;
 	char buffer[256]; /* For errors */
 
+	WARN;
 	if (!PyArg_ParseTuple(args, "s|i:atol", &s, &base))
 		return NULL;
 
@@ -809,7 +833,8 @@ strop_atol(PyObject *self, PyObject *args)
 	while (*end && isspace(Py_CHARMASK(*end)))
 		end++;
 	if (*end != '\0') {
-		sprintf(buffer, "invalid literal for atol(): %.200s", s);
+		PyOS_snprintf(buffer, sizeof(buffer),
+			      "invalid literal for atol(): %.200s", s);
 		PyErr_SetString(PyExc_ValueError, buffer);
 		Py_DECREF(x);
 		return NULL;
@@ -818,19 +843,19 @@ strop_atol(PyObject *self, PyObject *args)
 }
 
 
-static char atof__doc__[] =
-"atof(s) -> float\n\
-\n\
-Return the floating point number represented by the string s.";
+PyDoc_STRVAR(atof__doc__,
+"atof(s) -> float\n"
+"\n"
+"Return the floating point number represented by the string s.");
 
 static PyObject *
 strop_atof(PyObject *self, PyObject *args)
 {
-	extern double strtod(const char *, char **);
 	char *s, *end;
 	double x;
 	char buffer[256]; /* For errors */
 
+	WARN;
 	if (!PyArg_ParseTuple(args, "s:atof", &s))
 		return NULL;
 	while (*s && isspace(Py_CHARMASK(*s)))
@@ -841,17 +866,19 @@ strop_atof(PyObject *self, PyObject *args)
 	}
 	errno = 0;
 	PyFPE_START_PROTECT("strop_atof", return 0)
-	x = strtod(s, &end);
+	x = PyOS_ascii_strtod(s, &end);
 	PyFPE_END_PROTECT(x)
 	while (*end && isspace(Py_CHARMASK(*end)))
 		end++;
 	if (*end != '\0') {
-		sprintf(buffer, "invalid literal for atof(): %.200s", s);
+		PyOS_snprintf(buffer, sizeof(buffer),
+			      "invalid literal for atof(): %.200s", s);
 		PyErr_SetString(PyExc_ValueError, buffer);
 		return NULL;
 	}
 	else if (errno != 0) {
-		sprintf(buffer, "atof() literal too large: %.200s", s);
+		PyOS_snprintf(buffer, sizeof(buffer), 
+			      "atof() literal too large: %.200s", s);
 		PyErr_SetString(PyExc_ValueError, buffer);
 		return NULL;
 	}
@@ -859,18 +886,18 @@ strop_atof(PyObject *self, PyObject *args)
 }
 
 
-static char maketrans__doc__[] =
-"maketrans(frm, to) -> string\n\
-\n\
-Return a translation table (a string of 256 bytes long)\n\
-suitable for use in string.translate.  The strings frm and to\n\
-must be of the same length.";
+PyDoc_STRVAR(maketrans__doc__,
+"maketrans(frm, to) -> string\n"
+"\n"
+"Return a translation table (a string of 256 bytes long)\n"
+"suitable for use in string.translate.  The strings frm and to\n"
+"must be of the same length.");
 
 static PyObject *
 strop_maketrans(PyObject *self, PyObject *args)
 {
 	unsigned char *c, *from=NULL, *to=NULL;
-	int i, fromlen=0, tolen=0;
+	Py_ssize_t i, fromlen=0, tolen=0;
 	PyObject *result;
 
 	if (!PyArg_ParseTuple(args, "t#t#:maketrans", &from, &fromlen, &to, &tolen))
@@ -895,25 +922,27 @@ strop_maketrans(PyObject *self, PyObject *args)
 }
 
 
-static char translate__doc__[] =
-"translate(s,table [,deletechars]) -> string\n\
-\n\
-Return a copy of the string s, where all characters occurring\n\
-in the optional argument deletechars are removed, and the\n\
-remaining characters have been mapped through the given\n\
-translation table, which must be a string of length 256.";
+PyDoc_STRVAR(translate__doc__,
+"translate(s,table [,deletechars]) -> string\n"
+"\n"
+"Return a copy of the string s, where all characters occurring\n"
+"in the optional argument deletechars are removed, and the\n"
+"remaining characters have been mapped through the given\n"
+"translation table, which must be a string of length 256.");
 
 static PyObject *
 strop_translate(PyObject *self, PyObject *args)
 {
 	register char *input, *table, *output;
-	register int i, c, changed = 0;
+	Py_ssize_t i; 
+	int c, changed = 0;
 	PyObject *input_obj;
 	char *table1, *output_start, *del_table=NULL;
-	int inlen, tablen, dellen = 0;
+	Py_ssize_t inlen, tablen, dellen = 0;
 	PyObject *result;
 	int trans_table[256];
 
+	WARN;
 	if (!PyArg_ParseTuple(args, "St#|t#:translate", &input_obj,
 			      &table1, &tablen, &del_table, &dellen))
 		return NULL;
@@ -924,7 +953,7 @@ strop_translate(PyObject *self, PyObject *args)
 	}
 
 	table = table1;
-	inlen = PyString_Size(input_obj);
+	inlen = PyString_GET_SIZE(input_obj);
 	result = PyString_FromStringAndSize((char *)NULL, inlen);
 	if (result == NULL)
 		return NULL;
@@ -964,8 +993,8 @@ strop_translate(PyObject *self, PyObject *args)
 		return input_obj;
 	}
 	/* Fix the size of the resulting string */
-	if (inlen > 0 &&_PyString_Resize(&result, output-output_start))
-		return NULL;
+	if (inlen > 0)
+		_PyString_Resize(&result, output - output_start);
 	return result;
 }
 
@@ -982,10 +1011,10 @@ strop_translate(PyObject *self, PyObject *args)
   found, or -1 if not found.  If len of PAT is greater than length of
   MEM, the function returns -1.
 */
-static int
-mymemfind(const char *mem, int len, const char *pat, int pat_len)
+static Py_ssize_t 
+mymemfind(const char *mem, Py_ssize_t len, const char *pat, Py_ssize_t pat_len)
 {
-	register int ii;
+	register Py_ssize_t ii;
 
 	/* pattern can not occur in the last pat_len-1 chars */
 	len -= pat_len;
@@ -1007,11 +1036,11 @@ mymemfind(const char *mem, int len, const char *pat, int pat_len)
    meaning mem=1111 and pat==11 returns 2.
            mem=11111 and pat==11 also return 2.
  */
-static int
-mymemcnt(const char *mem, int len, const char *pat, int pat_len)
+static Py_ssize_t 
+mymemcnt(const char *mem, Py_ssize_t len, const char *pat, Py_ssize_t pat_len)
 {
-	register int offset = 0;
-	int nfound = 0;
+	register Py_ssize_t offset = 0;
+	Py_ssize_t nfound = 0;
 
 	while (len >= 0) {
 		offset = mymemfind(mem, len, pat, pat_len);
@@ -1044,15 +1073,15 @@ mymemcnt(const char *mem, int len, const char *pat, int pat_len)
        NULL if an error occurred.
 */
 static char *
-mymemreplace(const char *str, int len,		/* input string */
-             const char *pat, int pat_len,	/* pattern string to find */
-             const char *sub, int sub_len,	/* substitution string */
-             int count,				/* number of replacements */
-             int *out_len)
+mymemreplace(const char *str, Py_ssize_t len,		/* input string */
+             const char *pat, Py_ssize_t pat_len,	/* pattern string to find */
+             const char *sub, Py_ssize_t sub_len,	/* substitution string */
+             Py_ssize_t count,				/* number of replacements */
+	     Py_ssize_t *out_len)
 {
 	char *out_s;
 	char *new_s;
-	int nfound, offset, new_len;
+	Py_ssize_t nfound, offset, new_len;
 
 	if (len == 0 || pat_len > len)
 		goto return_same;
@@ -1060,7 +1089,7 @@ mymemreplace(const char *str, int len,		/* input string */
 	/* find length of output string */
 	nfound = mymemcnt(str, len, pat, pat_len);
 	if (count < 0)
-		count = INT_MAX;
+		count = PY_SSIZE_T_MAX;
 	else if (nfound > count)
 		nfound = count;
 	if (nfound == 0)
@@ -1110,22 +1139,23 @@ mymemreplace(const char *str, int len,		/* input string */
 }
 
 
-static char replace__doc__[] =
-"replace (str, old, new[, maxsplit]) -> string\n\
-\n\
-Return a copy of string str with all occurrences of substring\n\
-old replaced by new. If the optional argument maxsplit is\n\
-given, only the first maxsplit occurrences are replaced.";
+PyDoc_STRVAR(replace__doc__,
+"replace (str, old, new[, maxsplit]) -> string\n"
+"\n"
+"Return a copy of string str with all occurrences of substring\n"
+"old replaced by new. If the optional argument maxsplit is\n"
+"given, only the first maxsplit occurrences are replaced.");
 
 static PyObject *
 strop_replace(PyObject *self, PyObject *args)
 {
 	char *str, *pat,*sub,*new_s;
-	int len,pat_len,sub_len,out_len;
-	int count = -1;
-	PyObject *new;
+	Py_ssize_t len,pat_len,sub_len,out_len;
+	Py_ssize_t count = -1;
+	PyObject *newstr;
 
-	if (!PyArg_ParseTuple(args, "t#t#t#|i:replace",
+	WARN;
+	if (!PyArg_ParseTuple(args, "t#t#t#|n:replace",
 			      &str, &len, &pat, &pat_len, &sub, &sub_len,
 			      &count))
 		return NULL;
@@ -1146,14 +1176,14 @@ strop_replace(PyObject *self, PyObject *args)
 	}
 	if (out_len == -1) {
 		/* we're returning another reference to the input string */
-		new = PyTuple_GetItem(args, 0);
-		Py_XINCREF(new);
+		newstr = PyTuple_GetItem(args, 0);
+		Py_XINCREF(newstr);
 	}
 	else {
-		new = PyString_FromStringAndSize(new_s, out_len);
+		newstr = PyString_FromStringAndSize(new_s, out_len);
 		PyMem_FREE(new_s);
 	}
-	return new;
+	return newstr;
 }
 
 
@@ -1161,61 +1191,41 @@ strop_replace(PyObject *self, PyObject *args)
 
 static PyMethodDef
 strop_methods[] = {
-	{"atof",	strop_atof, 
-	 METH_VARARGS, atof__doc__},
-	{"atoi",	strop_atoi, 
-	 METH_VARARGS, atoi__doc__},
-	{"atol",	strop_atol, 
-	 METH_VARARGS, atol__doc__},
-	{"capitalize",	strop_capitalize, 
-	 METH_OLDARGS, capitalize__doc__},
-	{"count",	strop_count, 
-	 METH_VARARGS, count__doc__},
-	{"expandtabs",	strop_expandtabs, 
-	 METH_VARARGS, expandtabs__doc__},
-	{"find",	strop_find, 
-	 METH_VARARGS, find__doc__},
-	{"join",	strop_joinfields, 
-	 METH_VARARGS, joinfields__doc__},
-	{"joinfields",	strop_joinfields, 
-	 METH_VARARGS, joinfields__doc__},
-	{"lstrip",	strop_lstrip, 
-	 METH_OLDARGS, lstrip__doc__},
-	{"lower",	strop_lower, 
-	 METH_OLDARGS, lower__doc__},
-	{"maketrans",	strop_maketrans, 
-	 METH_VARARGS, maketrans__doc__},
-	{"replace",	strop_replace, 
-	 METH_VARARGS, replace__doc__},
-	{"rfind",	strop_rfind, 
-	 METH_VARARGS, rfind__doc__},
-	{"rstrip",	strop_rstrip, 
-	 METH_OLDARGS, rstrip__doc__},
-	{"split",	strop_splitfields, 
-	 METH_VARARGS, splitfields__doc__},
-	{"splitfields",	strop_splitfields, 
-	 METH_VARARGS, splitfields__doc__},
-	{"strip",	strop_strip, 
-	 METH_OLDARGS, strip__doc__},
-	{"swapcase",	strop_swapcase, 
-	 METH_OLDARGS, swapcase__doc__},
-	{"translate",	strop_translate, 
-	 METH_VARARGS, translate__doc__},
-	{"upper",	strop_upper, 
-	 METH_OLDARGS, upper__doc__},
+	{"atof",	strop_atof,	   METH_VARARGS, atof__doc__},
+	{"atoi",	strop_atoi,	   METH_VARARGS, atoi__doc__},
+	{"atol",	strop_atol,	   METH_VARARGS, atol__doc__},
+	{"capitalize",	strop_capitalize,  METH_O,       capitalize__doc__},
+	{"count",	strop_count,	   METH_VARARGS, count__doc__},
+	{"expandtabs",	strop_expandtabs,  METH_VARARGS, expandtabs__doc__},
+	{"find",	strop_find,	   METH_VARARGS, find__doc__},
+	{"join",	strop_joinfields,  METH_VARARGS, joinfields__doc__},
+	{"joinfields",	strop_joinfields,  METH_VARARGS, joinfields__doc__},
+	{"lstrip",	strop_lstrip,	   METH_O,       lstrip__doc__},
+	{"lower",	strop_lower,	   METH_O,       lower__doc__},
+	{"maketrans",	strop_maketrans,   METH_VARARGS, maketrans__doc__},
+	{"replace",	strop_replace,	   METH_VARARGS, replace__doc__},
+	{"rfind",	strop_rfind,	   METH_VARARGS, rfind__doc__},
+	{"rstrip",	strop_rstrip,	   METH_O,       rstrip__doc__},
+	{"split",	strop_splitfields, METH_VARARGS, splitfields__doc__},
+	{"splitfields",	strop_splitfields, METH_VARARGS, splitfields__doc__},
+	{"strip",	strop_strip,	   METH_O,       strip__doc__},
+	{"swapcase",	strop_swapcase,    METH_O,       swapcase__doc__},
+	{"translate",	strop_translate,   METH_VARARGS, translate__doc__},
+	{"upper",	strop_upper,	   METH_O,       upper__doc__},
 	{NULL,		NULL}	/* sentinel */
 };
 
 
-DL_EXPORT(void)
+PyMODINIT_FUNC
 initstrop(void)
 {
-	PyObject *m, *d, *s;
+	PyObject *m, *s;
 	char buf[256];
 	int c, n;
 	m = Py_InitModule4("strop", strop_methods, strop_module__doc__,
 			   (PyObject*)NULL, PYTHON_API_VERSION);
-	d = PyModule_GetDict(m);
+	if (m == NULL)
+		return;
 
 	/* Create 'whitespace' object */
 	n = 0;
@@ -1224,10 +1234,9 @@ initstrop(void)
 			buf[n++] = c;
 	}
 	s = PyString_FromStringAndSize(buf, n);
-	if (s) {
-		PyDict_SetItemString(d, "whitespace", s);
-		Py_DECREF(s);
-	}
+	if (s)
+		PyModule_AddObject(m, "whitespace", s);
+
 	/* Create 'lowercase' object */
 	n = 0;
 	for (c = 0; c < 256; c++) {
@@ -1235,10 +1244,8 @@ initstrop(void)
 			buf[n++] = c;
 	}
 	s = PyString_FromStringAndSize(buf, n);
-	if (s) {
-		PyDict_SetItemString(d, "lowercase", s);
-		Py_DECREF(s);
-	}
+	if (s)
+		PyModule_AddObject(m, "lowercase", s);
 
 	/* Create 'uppercase' object */
 	n = 0;
@@ -1247,8 +1254,6 @@ initstrop(void)
 			buf[n++] = c;
 	}
 	s = PyString_FromStringAndSize(buf, n);
-	if (s) {
-		PyDict_SetItemString(d, "uppercase", s);
-		Py_DECREF(s);
-	}
+	if (s)
+		PyModule_AddObject(m, "uppercase", s);
 }
